# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/deployment.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/deployment.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from ..google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18gateway/deployment.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a\x1egoogle/protobuf/duration.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xc9\x1a\n\nDeployment\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12!\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x0b\xe2\x41\x01\x01\xbaH\x04r\x02\x18?\x12!\n\x0b\x64\x65scription\x18\x08 \x01(\tB\x0c\xe2\x41\x01\x01\xbaH\x05r\x03\x18\xe7\x07\x12\x36\n\x0b\x63reate_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0b\x65xpire_time\x18\x38 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x01\x12\x35\n\npurge_time\x18\x36 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x36\n\x0b\x64\x65lete_time\x18= \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x19\n\ncreated_by\x18\x1d \x01(\tB\x05\xe2\x41\x02\x03\x05\x12/\n\x05state\x18\x05 \x01(\x0e\x32\x19.gateway.Deployment.StateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x06 \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12U\n\x0b\x61nnotations\x18@ \x03(\x0b\x32$.gateway.Deployment.AnnotationsEntryB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12+\n\x11min_replica_count\x18\r \x01(\x05\x42\x10\xe2\x41\x02\x01\x05\xbaH\x04\x1a\x02(\x00\xa8\xf1\x04\x01\x12.\n\x11max_replica_count\x18\x0e \x01(\x05\x42\x13\xe2\x41\x02\x01\x05\xbaH\x07\x1a\x05\x18\xe8\x07(\x00\xa8\xf1\x04\x01\x12 \n\rreplica_count\x18# \x01(\x05\x42\t\xe2\x41\x02\x03\x05\xa8\xf1\x04\x01\x12=\n\x12\x61utoscaling_policy\x18& \x01(\x0b\x32\x1a.gateway.AutoscalingPolicyB\x05\xe2\x41\x02\x01\x05\x12\x19\n\nbase_model\x18\x12 \x01(\tB\x05\xe2\x41\x02\x02\x05\x12&\n\x11\x61\x63\x63\x65lerator_count\x18* \x01(\x05\x42\x0b\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\x12\x38\n\x10\x61\x63\x63\x65lerator_type\x18+ \x01(\x0e\x32\x18.gateway.AcceleratorTypeB\x04\xe2\x41\x01\x01\x12\x36\n\tprecision\x18. \x01(\x0e\x32\x1d.gateway.Deployment.PrecisionB\x04\xe2\x41\x01\x01\x12\x35\n\nworld_size\x18\x18 \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12:\n\x0fgenerator_count\x18\x1a \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12H\n\x1b\x64isaggregated_prefill_count\x18\x13 \x01(\x05\x42#\xe2\x41\x01\x01\xbaH\x06\x1a\x04\x18\x10(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12K\n disaggregated_prefill_world_size\x18\x19 \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12=\n\x0emax_batch_size\x18\x14 \x01(\x05\x42%\xe2\x41\x02\x01\x05\xbaH\x07\x1a\x05\x18\x80\x08(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x0f\n\x07\x63luster\x18\x34 \x01(\t\x12\x1f\n\renable_addons\x18\x31 \x01(\x08\x42\x08\xe2\x41\x01\x01\xa8\xf1\x04\x01\x12.\n\nlive_merge\x18; \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12&\n\x11\x64raft_token_count\x18\x1b \x01(\x05\x42\x0b\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\x12\x19\n\x0b\x64raft_model\x18\" \x01(\tB\x04\xe2\x41\x01\x01\x12&\n\x18ngram_speculation_length\x18\x32 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12>\n\x13max_peft_batch_size\x18\x1e \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12@\n\x13kv_cache_memory_pct\x18\x1f \x01(\x05\x42#\xe2\x41\x01\x01\xbaH\x06\x1a\x04\x18Z(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12<\n\x17\x65nable_session_affinity\x18  \x01(\x08\x42\x1b\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x11\x12\x0f\x45NTERPRISE_ONLY\x12:\n\x15\x64irect_route_api_keys\x18, \x03(\tB\x1b\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x11\x12\x0f\x45NTERPRISE_ONLY\x12-\n\timage_tag\x18! \x01(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12+\n\x16num_peft_device_cached\x18\x30 \x01(\x05\x42\x0b\xe2\x41\x01\x03\xbaH\x04\x1a\x02(\x00\x12P\n\x11\x64irect_route_type\x18\x35 \x01(\x0e\x32\x18.gateway.DirectRouteTypeB\x1b\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x11\x12\x0f\x45NTERPRISE_ONLY\x12\x39\n\x13\x64irect_route_handle\x18\x39 \x01(\tB\x1c\xe2\x41\x02\x01\x03\xfa\xd2\xe4\x93\x02\x11\x12\x0f\x45NTERPRISE_ONLY\x12#\n\x13\x64\x65ployment_template\x18\' \x01(\tB\x04\xe2\x41\x01\x01H\x00\x12,\n\tauto_tune\x18( \x01(\x0b\x32\x11.gateway.AutoTuneB\x04\xe2\x41\x01\x01H\x00\x12&\n\x06region\x18\x33 \x01(\x0e\x32\x0f.gateway.RegionB\x05\xe2\x41\x02\x01\x05\x12\x36\n\x12\x64isable_accounting\x18\x37 \x01(\x08\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12.\n\nextra_args\x18: \x03(\tB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12=\n\x12max_context_length\x18\x43 \x01(\x05\x42!\xe2\x41\x01\x01\xbaH\x04\x1a\x02(\x00\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12V\n\x0c\x65xtra_values\x18< \x03(\x0b\x32$.gateway.Deployment.ExtraValuesEntryB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x30\n\x06\x65ngine\x18\x42 \x01(\x0e\x32\x1a.gateway.Deployment.EngineB\x04\xe2\x41\x01\x01\x12\x35\n\x0bupdate_time\x18? \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x1a\n\x0c\x66or_training\x18\x41 \x01(\x08\x42\x04\xe2\x41\x01\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x45xtraValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"l\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\t\n\x05READY\x10\x02\x12\x0c\n\x08\x44\x45LETING\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\x0c\n\x08UPDATING\x10\x05\x12\x0b\n\x07\x44\x45LETED\x10\x06\"\xac\x01\n\tPrecision\x12\x19\n\x15PRECISION_UNSPECIFIED\x10\x00\x12\x08\n\x04\x46P16\x10\x01\x12\x07\n\x03\x46P8\x10\x02\x12\n\n\x06\x46P8_MM\x10\x03\x12\n\n\x06\x46P8_AR\x10\x04\x12\x12\n\x0e\x46P8_MM_KV_ATTN\x10\x05\x12\n\n\x06\x46P8_KV\x10\x06\x12\r\n\tFP8_MM_V2\x10\x07\x12\n\n\x06\x46P8_V2\x10\x08\x12\x15\n\x11\x46P8_MM_KV_ATTN_V2\x10\t\x12\x07\n\x03NF4\x10\n\"F\n\x06\x45ngine\x12\x16\n\x12\x45NGINE_UNSPECIFIED\x10\x00\x12\x11\n\rFIREATTENTION\x10\x01\x12\x08\n\x04VLLM\x10\x02\x12\x07\n\x03NIM\x10\x03:\xef\x03\xea\x41g\n\x1b\x61pi.fireworks.ai/Deployment\x12/accounts/{AccountId}/deployments/{DeploymentId}*\x0b\x64\x65ployments2\ndeployment\xbaH\xf2\x02\x1a\xc9\x01\n:deployment.max_replica_count.larger_than_min_replica_count\x12:min_replica_count cannot be greater than max_replica_count\x1aOthis.max_replica_count == 0 || this.min_replica_count <= this.max_replica_count\x1a\xa3\x01\n>deployment.max_peft_batch_size.not_greater_than_max_batch_size\x12\x30max_peft_batch_size cannot exceed max_batch_size\x1a/this.max_peft_batch_size <= this.max_batch_size\x82\xf1\x04\x0b\n\x07\x41\x63\x63ount\x18\x01\x42\x0b\n\tpreconfig\"%\n\x08\x41utoTune\x12\x19\n\x0blong_prompt\x18\x01 \x01(\x08\x42\x04\xe2\x41\x01\x01\"\xfc\x03\n\x11\x41utoscalingPolicy\x12@\n\x0fscale_up_window\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xe2\x41\x01\x01\xbaH\x05\xaa\x01\x02\x32\x00\x12\x42\n\x11scale_down_window\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xe2\x41\x01\x01\xbaH\x05\xaa\x01\x02\x32\x00\x12\xe3\x01\n\x14scale_to_zero_window\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\xa9\x01\xe2\x41\x01\x01\xbaH\xa1\x01\xba\x01\x9d\x01\n8autoscaling_policy.scale_to_zero_window.duration.minimum\x12/scale_to_zero_window must be at least 5 minutes\x1a\x30this == duration(\'0s\') || this >= duration(\'5m\')\x12G\n\x0cload_targets\x18\x04 \x03(\x0b\x32+.gateway.AutoscalingPolicy.LoadTargetsEntryB\x04\xe2\x41\x01\x01\x1a\x32\n\x10LoadTargetsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x02:\x02\x38\x01\"\xca\x01\n\x17\x43reateDeploymentRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12-\n\ndeployment\x18\x02 \x01(\x0b\x32\x13.gateway.DeploymentB\x04\xe2\x41\x01\x02\x12!\n\x13\x64isable_auto_deploy\x18\x05 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12*\n\x1c\x64isable_speculative_decoding\x18\x06 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12\x1b\n\rdeployment_id\x18\x07 \x01(\tB\x04\xe2\x41\x01\x01\"*\n\x14GetDeploymentRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\"\xc2\x01\n\x16ListDeploymentsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x0cshow_deleted\x18\x06 \x01(\x08\x12\x1b\n\rshow_internal\x18\x07 \x01(\x08\x42\x04\xe2\x41\x01\x01\"p\n\x17ListDeploymentsResponse\x12(\n\x0b\x64\x65ployments\x18\x01 \x03(\x0b\x32\x13.gateway.Deployment\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"\x7f\n\x17UpdateDeploymentRequest\x12-\n\ndeployment\x18\x01 \x01(\x0b\x32\x13.gateway.DeploymentB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"^\n\x17\x44\x65leteDeploymentRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x12\n\x04hard\x18\x02 \x01(\x08\x42\x04\xe2\x41\x01\x01\x12\x1b\n\rignore_checks\x18\x03 \x01(\x08\x42\x04\xe2\x41\x01\x01\"/\n\x19UndeleteDeploymentRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\"I\n\x16ScaleDeploymentRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x1b\n\rreplica_count\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\"E\n\x1bGetDeploymentMetricsRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x12\n\ntime_range\x18\x02 \x01(\t\"\x93\x01\n\x1cGetDeploymentMetricsResponse\x12\x43\n\x07metrics\x18\x01 \x03(\x0b\x32\x32.gateway.GetDeploymentMetricsResponse.MetricsEntry\x1a.\n\x0cMetricsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"M\n!GetDeploymentPrerequisitesRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x12\n\x04name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02\"\\\n\x17\x44\x65ploymentPrerequisites\x12\x41\n\x13\x61\x63\x63\x65lerator_configs\x18\x01 \x03(\x0b\x32$.gateway.DeploymentAcceleratorConfig\"\xc4\x01\n\x1b\x44\x65ploymentAcceleratorConfig\x12\x32\n\x10\x61\x63\x63\x65lerator_type\x18\x01 \x01(\x0e\x32\x18.gateway.AcceleratorType\x12\x30\n\tprecision\x18\x02 \x01(\x0e\x32\x1d.gateway.Deployment.Precision\x12\x1d\n\x15min_accelerator_count\x18\x03 \x01(\x05\x12 \n\x07regions\x18\x04 \x03(\x0e\x32\x0f.gateway.Region*\xf0\x02\n\x06Region\x12\x16\n\x12REGION_UNSPECIFIED\x10\x00\x12\r\n\tUS_IOWA_1\x10\x01\x12\x11\n\rUS_VIRGINIA_1\x10\x02\x12\x11\n\rUS_VIRGINIA_2\x10\x03\x12\x11\n\rUS_ILLINOIS_1\x10\x04\x12\x0e\n\nAP_TOKYO_1\x10\x05\x12\x13\n\x0b\x45U_LONDON_1\x10\x06\x1a\x02\x08\x01\x12\x10\n\x0cUS_ARIZONA_1\x10\x07\x12\x0e\n\nUS_TEXAS_1\x10\x08\x12\x11\n\rUS_ILLINOIS_2\x10\t\x12\x12\n\x0e\x45U_FRANKFURT_1\x10\n\x12\x0e\n\nUS_TEXAS_2\x10\x0b\x12\x12\n\nEU_PARIS_1\x10\x0c\x1a\x02\x08\x01\x12\x11\n\rEU_HELSINKI_1\x10\x0e\x12\x13\n\x0bUS_NEVADA_1\x10\x0f\x1a\x02\x08\x01\x12\x10\n\x0c\x45U_ICELAND_1\x10\x10\x12\x10\n\x0c\x45U_ICELAND_2\x10\x11\x12\x13\n\x0fUS_WASHINGTON_1\x10\x12\x12\x13\n\x0fUS_WASHINGTON_2\x10\x13*\xe3\x01\n\x0f\x41\x63\x63\x65leratorType\x12 \n\x1c\x41\x43\x43\x45LERATOR_TYPE_UNSPECIFIED\x10\x00\x12\x14\n\x10NVIDIA_A100_80GB\x10\x01\x12\x14\n\x10NVIDIA_H100_80GB\x10\x02\x12\x14\n\x10\x41MD_MI300X_192GB\x10\x03\x12\x14\n\x10NVIDIA_A10G_24GB\x10\x04\x12\x14\n\x10NVIDIA_A100_40GB\x10\x05\x12\x12\n\x0eNVIDIA_L4_24GB\x10\x06\x12\x15\n\x11NVIDIA_H200_141GB\x10\x07\x12\x15\n\x11NVIDIA_B200_180GB\x10\x08*x\n\x0f\x44irectRouteType\x12!\n\x1d\x44IRECT_ROUTE_TYPE_UNSPECIFIED\x10\x00\x12\x0c\n\x08INTERNET\x10\x01\x12\x1f\n\x1bGCP_PRIVATE_SERVICE_CONNECT\x10\x02\x12\x13\n\x0f\x41WS_PRIVATELINK\x10\x03\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.deployment_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_REGION'].values_by_name["EU_LONDON_1"]._loaded_options = None
  _globals['_REGION'].values_by_name["EU_LONDON_1"]._serialized_options = b'\010\001'
  _globals['_REGION'].values_by_name["EU_PARIS_1"]._loaded_options = None
  _globals['_REGION'].values_by_name["EU_PARIS_1"]._serialized_options = b'\010\001'
  _globals['_REGION'].values_by_name["US_NEVADA_1"]._loaded_options = None
  _globals['_REGION'].values_by_name["US_NEVADA_1"]._serialized_options = b'\010\001'
  _globals['_DEPLOYMENT_ANNOTATIONSENTRY']._loaded_options = None
  _globals['_DEPLOYMENT_ANNOTATIONSENTRY']._serialized_options = b'8\001'
  _globals['_DEPLOYMENT_EXTRAVALUESENTRY']._loaded_options = None
  _globals['_DEPLOYMENT_EXTRAVALUESENTRY']._serialized_options = b'8\001'
  _globals['_DEPLOYMENT'].fields_by_name['name']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['display_name']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['display_name']._serialized_options = b'\342A\001\001\272H\004r\002\030?'
  _globals['_DEPLOYMENT'].fields_by_name['description']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['description']._serialized_options = b'\342A\001\001\272H\005r\003\030\347\007'
  _globals['_DEPLOYMENT'].fields_by_name['create_time']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['expire_time']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['expire_time']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['purge_time']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['purge_time']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['delete_time']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['delete_time']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['created_by']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['state']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['status']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENT'].fields_by_name['annotations']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['annotations']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['min_replica_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['min_replica_count']._serialized_options = b'\342A\002\001\005\272H\004\032\002(\000\250\361\004\001'
  _globals['_DEPLOYMENT'].fields_by_name['max_replica_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['max_replica_count']._serialized_options = b'\342A\002\001\005\272H\007\032\005\030\350\007(\000\250\361\004\001'
  _globals['_DEPLOYMENT'].fields_by_name['replica_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['replica_count']._serialized_options = b'\342A\002\003\005\250\361\004\001'
  _globals['_DEPLOYMENT'].fields_by_name['autoscaling_policy']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['autoscaling_policy']._serialized_options = b'\342A\002\001\005'
  _globals['_DEPLOYMENT'].fields_by_name['base_model']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['base_model']._serialized_options = b'\342A\002\002\005'
  _globals['_DEPLOYMENT'].fields_by_name['accelerator_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['accelerator_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000'
  _globals['_DEPLOYMENT'].fields_by_name['accelerator_type']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['accelerator_type']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['precision']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['precision']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['world_size']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['world_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['generator_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['generator_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['disaggregated_prefill_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['disaggregated_prefill_count']._serialized_options = b'\342A\001\001\272H\006\032\004\030\020(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['disaggregated_prefill_world_size']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['disaggregated_prefill_world_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['max_batch_size']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['max_batch_size']._serialized_options = b'\342A\002\001\005\272H\007\032\005\030\200\010(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['enable_addons']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['enable_addons']._serialized_options = b'\342A\001\001\250\361\004\001'
  _globals['_DEPLOYMENT'].fields_by_name['live_merge']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['live_merge']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['draft_token_count']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['draft_token_count']._serialized_options = b'\342A\001\001\272H\004\032\002(\000'
  _globals['_DEPLOYMENT'].fields_by_name['draft_model']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['draft_model']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['ngram_speculation_length']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['ngram_speculation_length']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['max_peft_batch_size']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['max_peft_batch_size']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['kv_cache_memory_pct']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['kv_cache_memory_pct']._serialized_options = b'\342A\001\001\272H\006\032\004\030Z(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['enable_session_affinity']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['enable_session_affinity']._serialized_options = b'\342A\001\001\372\322\344\223\002\021\022\017ENTERPRISE_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_api_keys']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_api_keys']._serialized_options = b'\342A\001\001\372\322\344\223\002\021\022\017ENTERPRISE_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['image_tag']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['image_tag']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['num_peft_device_cached']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['num_peft_device_cached']._serialized_options = b'\342A\001\003\272H\004\032\002(\000'
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_type']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_type']._serialized_options = b'\342A\001\001\372\322\344\223\002\021\022\017ENTERPRISE_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_handle']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['direct_route_handle']._serialized_options = b'\342A\002\001\003\372\322\344\223\002\021\022\017ENTERPRISE_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['deployment_template']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['deployment_template']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['auto_tune']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['auto_tune']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['region']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['region']._serialized_options = b'\342A\002\001\005'
  _globals['_DEPLOYMENT'].fields_by_name['disable_accounting']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['disable_accounting']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['extra_args']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['extra_args']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['max_context_length']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['max_context_length']._serialized_options = b'\342A\001\001\272H\004\032\002(\000\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['extra_values']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['extra_values']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_DEPLOYMENT'].fields_by_name['engine']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['engine']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT'].fields_by_name['update_time']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_DEPLOYMENT'].fields_by_name['for_training']._loaded_options = None
  _globals['_DEPLOYMENT'].fields_by_name['for_training']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENT']._loaded_options = None
  _globals['_DEPLOYMENT']._serialized_options = b'\352Ag\n\033api.fireworks.ai/Deployment\022/accounts/{AccountId}/deployments/{DeploymentId}*\013deployments2\ndeployment\272H\362\002\032\311\001\n:deployment.max_replica_count.larger_than_min_replica_count\022:min_replica_count cannot be greater than max_replica_count\032Othis.max_replica_count == 0 || this.min_replica_count <= this.max_replica_count\032\243\001\n>deployment.max_peft_batch_size.not_greater_than_max_batch_size\0220max_peft_batch_size cannot exceed max_batch_size\032/this.max_peft_batch_size <= this.max_batch_size\202\361\004\013\n\007Account\030\001'
  _globals['_AUTOTUNE'].fields_by_name['long_prompt']._loaded_options = None
  _globals['_AUTOTUNE'].fields_by_name['long_prompt']._serialized_options = b'\342A\001\001'
  _globals['_AUTOSCALINGPOLICY_LOADTARGETSENTRY']._loaded_options = None
  _globals['_AUTOSCALINGPOLICY_LOADTARGETSENTRY']._serialized_options = b'8\001'
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_up_window']._loaded_options = None
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_up_window']._serialized_options = b'\342A\001\001\272H\005\252\001\0022\000'
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_down_window']._loaded_options = None
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_down_window']._serialized_options = b'\342A\001\001\272H\005\252\001\0022\000'
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_to_zero_window']._loaded_options = None
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['scale_to_zero_window']._serialized_options = b'\342A\001\001\272H\241\001\272\001\235\001\n8autoscaling_policy.scale_to_zero_window.duration.minimum\022/scale_to_zero_window must be at least 5 minutes\0320this == duration(\'0s\') || this >= duration(\'5m\')'
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['load_targets']._loaded_options = None
  _globals['_AUTOSCALINGPOLICY'].fields_by_name['load_targets']._serialized_options = b'\342A\001\001'
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['deployment']._loaded_options = None
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['deployment']._serialized_options = b'\342A\001\002'
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['disable_auto_deploy']._loaded_options = None
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['disable_auto_deploy']._serialized_options = b'\342A\001\001'
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['disable_speculative_decoding']._loaded_options = None
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['disable_speculative_decoding']._serialized_options = b'\342A\001\001'
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['deployment_id']._loaded_options = None
  _globals['_CREATEDEPLOYMENTREQUEST'].fields_by_name['deployment_id']._serialized_options = b'\342A\001\001'
  _globals['_GETDEPLOYMENTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETDEPLOYMENTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['show_internal']._loaded_options = None
  _globals['_LISTDEPLOYMENTSREQUEST'].fields_by_name['show_internal']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEDEPLOYMENTREQUEST'].fields_by_name['deployment']._loaded_options = None
  _globals['_UPDATEDEPLOYMENTREQUEST'].fields_by_name['deployment']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEDEPLOYMENTREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEDEPLOYMENTREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['hard']._loaded_options = None
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['hard']._serialized_options = b'\342A\001\001'
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['ignore_checks']._loaded_options = None
  _globals['_DELETEDEPLOYMENTREQUEST'].fields_by_name['ignore_checks']._serialized_options = b'\342A\001\001'
  _globals['_UNDELETEDEPLOYMENTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_UNDELETEDEPLOYMENTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_SCALEDEPLOYMENTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_SCALEDEPLOYMENTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_SCALEDEPLOYMENTREQUEST'].fields_by_name['replica_count']._loaded_options = None
  _globals['_SCALEDEPLOYMENTREQUEST'].fields_by_name['replica_count']._serialized_options = b'\342A\001\001'
  _globals['_GETDEPLOYMENTMETRICSREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETDEPLOYMENTMETRICSREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_GETDEPLOYMENTMETRICSRESPONSE_METRICSENTRY']._loaded_options = None
  _globals['_GETDEPLOYMENTMETRICSRESPONSE_METRICSENTRY']._serialized_options = b'8\001'
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_REGION']._serialized_start=5756
  _globals['_REGION']._serialized_end=6124
  _globals['_ACCELERATORTYPE']._serialized_start=6127
  _globals['_ACCELERATORTYPE']._serialized_end=6354
  _globals['_DIRECTROUTETYPE']._serialized_start=6356
  _globals['_DIRECTROUTETYPE']._serialized_end=6476
  _globals['_DEPLOYMENT']._serialized_start=300
  _globals['_DEPLOYMENT']._serialized_end=3701
  _globals['_DEPLOYMENT_ANNOTATIONSENTRY']._serialized_start=2731
  _globals['_DEPLOYMENT_ANNOTATIONSENTRY']._serialized_end=2781
  _globals['_DEPLOYMENT_EXTRAVALUESENTRY']._serialized_start=2783
  _globals['_DEPLOYMENT_EXTRAVALUESENTRY']._serialized_end=2833
  _globals['_DEPLOYMENT_STATE']._serialized_start=2835
  _globals['_DEPLOYMENT_STATE']._serialized_end=2943
  _globals['_DEPLOYMENT_PRECISION']._serialized_start=2946
  _globals['_DEPLOYMENT_PRECISION']._serialized_end=3118
  _globals['_DEPLOYMENT_ENGINE']._serialized_start=3120
  _globals['_DEPLOYMENT_ENGINE']._serialized_end=3190
  _globals['_AUTOTUNE']._serialized_start=3703
  _globals['_AUTOTUNE']._serialized_end=3740
  _globals['_AUTOSCALINGPOLICY']._serialized_start=3743
  _globals['_AUTOSCALINGPOLICY']._serialized_end=4251
  _globals['_AUTOSCALINGPOLICY_LOADTARGETSENTRY']._serialized_start=4201
  _globals['_AUTOSCALINGPOLICY_LOADTARGETSENTRY']._serialized_end=4251
  _globals['_CREATEDEPLOYMENTREQUEST']._serialized_start=4254
  _globals['_CREATEDEPLOYMENTREQUEST']._serialized_end=4456
  _globals['_GETDEPLOYMENTREQUEST']._serialized_start=4458
  _globals['_GETDEPLOYMENTREQUEST']._serialized_end=4500
  _globals['_LISTDEPLOYMENTSREQUEST']._serialized_start=4503
  _globals['_LISTDEPLOYMENTSREQUEST']._serialized_end=4697
  _globals['_LISTDEPLOYMENTSRESPONSE']._serialized_start=4699
  _globals['_LISTDEPLOYMENTSRESPONSE']._serialized_end=4811
  _globals['_UPDATEDEPLOYMENTREQUEST']._serialized_start=4813
  _globals['_UPDATEDEPLOYMENTREQUEST']._serialized_end=4940
  _globals['_DELETEDEPLOYMENTREQUEST']._serialized_start=4942
  _globals['_DELETEDEPLOYMENTREQUEST']._serialized_end=5036
  _globals['_UNDELETEDEPLOYMENTREQUEST']._serialized_start=5038
  _globals['_UNDELETEDEPLOYMENTREQUEST']._serialized_end=5085
  _globals['_SCALEDEPLOYMENTREQUEST']._serialized_start=5087
  _globals['_SCALEDEPLOYMENTREQUEST']._serialized_end=5160
  _globals['_GETDEPLOYMENTMETRICSREQUEST']._serialized_start=5162
  _globals['_GETDEPLOYMENTMETRICSREQUEST']._serialized_end=5231
  _globals['_GETDEPLOYMENTMETRICSRESPONSE']._serialized_start=5234
  _globals['_GETDEPLOYMENTMETRICSRESPONSE']._serialized_end=5381
  _globals['_GETDEPLOYMENTMETRICSRESPONSE_METRICSENTRY']._serialized_start=5335
  _globals['_GETDEPLOYMENTMETRICSRESPONSE_METRICSENTRY']._serialized_end=5381
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST']._serialized_start=5383
  _globals['_GETDEPLOYMENTPREREQUISITESREQUEST']._serialized_end=5460
  _globals['_DEPLOYMENTPREREQUISITES']._serialized_start=5462
  _globals['_DEPLOYMENTPREREQUISITES']._serialized_end=5554
  _globals['_DEPLOYMENTACCELERATORCONFIG']._serialized_start=5557
  _globals['_DEPLOYMENTACCELERATORCONFIG']._serialized_end=5753
# @@protoc_insertion_point(module_scope)



from . import options_pb2 as _options_pb2
from . import status_pb2 as _status_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from ..google.api import resource_pb2 as _resource_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Environment(_message.Message):
    __slots__ = ("name", "display_name", "create_time", "created_by", "state", "status", "connection", "base_image_ref", "image_ref", "snapshot_image_ref", "shared", "annotations", "update_time")
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATE_UNSPECIFIED: _ClassVar[Environment.State]
        CREATING: _ClassVar[Environment.State]
        DISCONNECTED: _ClassVar[Environment.State]
        CONNECTING: _ClassVar[Environment.State]
        CONNECTED: _ClassVar[Environment.State]
        DISCONNECTING: _ClassVar[Environment.State]
        RECONNECTING: _ClassVar[Environment.State]
        DELETING: _ClassVar[Environment.State]
    STATE_UNSPECIFIED: Environment.State
    CREATING: Environment.State
    DISCONNECTED: Environment.State
    CONNECTING: Environment.State
    CONNECTED: Environment.State
    DISCONNECTING: Environment.State
    RECONNECTING: Environment.State
    DELETING: Environment.State
    class AnnotationsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_FIELD_NUMBER: _ClassVar[int]
    BASE_IMAGE_REF_FIELD_NUMBER: _ClassVar[int]
    IMAGE_REF_FIELD_NUMBER: _ClassVar[int]
    SNAPSHOT_IMAGE_REF_FIELD_NUMBER: _ClassVar[int]
    SHARED_FIELD_NUMBER: _ClassVar[int]
    ANNOTATIONS_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    create_time: _timestamp_pb2.Timestamp
    created_by: str
    state: Environment.State
    status: _status_pb2.Status
    connection: EnvironmentConnection
    base_image_ref: str
    image_ref: str
    snapshot_image_ref: str
    shared: bool
    annotations: _containers.ScalarMap[str, str]
    update_time: _timestamp_pb2.Timestamp
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., state: _Optional[_Union[Environment.State, str]] = ..., status: _Optional[_Union[_status_pb2.Status, _Mapping]] = ..., connection: _Optional[_Union[EnvironmentConnection, _Mapping]] = ..., base_image_ref: _Optional[str] = ..., image_ref: _Optional[str] = ..., snapshot_image_ref: _Optional[str] = ..., shared: bool = ..., annotations: _Optional[_Mapping[str, str]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class EnvironmentConnection(_message.Message):
    __slots__ = ("node_pool_id", "num_ranks", "role", "zone", "use_local_storage")
    NODE_POOL_ID_FIELD_NUMBER: _ClassVar[int]
    NUM_RANKS_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    ZONE_FIELD_NUMBER: _ClassVar[int]
    USE_LOCAL_STORAGE_FIELD_NUMBER: _ClassVar[int]
    node_pool_id: str
    num_ranks: int
    role: str
    zone: str
    use_local_storage: bool
    def __init__(self, node_pool_id: _Optional[str] = ..., num_ranks: _Optional[int] = ..., role: _Optional[str] = ..., zone: _Optional[str] = ..., use_local_storage: bool = ...) -> None: ...

class CreateEnvironmentRequest(_message.Message):
    __slots__ = ("parent", "environment", "environment_id")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    parent: str
    environment: Environment
    environment_id: str
    def __init__(self, parent: _Optional[str] = ..., environment: _Optional[_Union[Environment, _Mapping]] = ..., environment_id: _Optional[str] = ...) -> None: ...

class GetEnvironmentRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ListEnvironmentsRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter", "order_by")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ...) -> None: ...

class ListEnvironmentsResponse(_message.Message):
    __slots__ = ("environments", "next_page_token", "total_size")
    ENVIRONMENTS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    environments: _containers.RepeatedCompositeFieldContainer[Environment]
    next_page_token: str
    total_size: int
    def __init__(self, environments: _Optional[_Iterable[_Union[Environment, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class UpdateEnvironmentRequest(_message.Message):
    __slots__ = ("environment", "update_mask")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    environment: Environment
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, environment: _Optional[_Union[Environment, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class DeleteEnvironmentRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class BatchDeleteEnvironmentsRequest(_message.Message):
    __slots__ = ("parent", "names")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    NAMES_FIELD_NUMBER: _ClassVar[int]
    parent: str
    names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, parent: _Optional[str] = ..., names: _Optional[_Iterable[str]] = ...) -> None: ...

class ConnectEnvironmentRequest(_message.Message):
    __slots__ = ("name", "connection", "vscode_version")
    NAME_FIELD_NUMBER: _ClassVar[int]
    CONNECTION_FIELD_NUMBER: _ClassVar[int]
    VSCODE_VERSION_FIELD_NUMBER: _ClassVar[int]
    name: str
    connection: EnvironmentConnection
    vscode_version: str
    def __init__(self, name: _Optional[str] = ..., connection: _Optional[_Union[EnvironmentConnection, _Mapping]] = ..., vscode_version: _Optional[str] = ...) -> None: ...

class DisconnectEnvironmentRequest(_message.Message):
    __slots__ = ("name", "force", "reset_snapshots")
    NAME_FIELD_NUMBER: _ClassVar[int]
    FORCE_FIELD_NUMBER: _ClassVar[int]
    RESET_SNAPSHOTS_FIELD_NUMBER: _ClassVar[int]
    name: str
    force: bool
    reset_snapshots: bool
    def __init__(self, name: _Optional[str] = ..., force: bool = ..., reset_snapshots: bool = ...) -> None: ...



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: google/longrunning/operations.proto
# plugin: python-betterproto
# This file has been @generated

__all__ = (
    "Operation",
    "GetOperationRequest",
)


from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class Operation(betterproto.Message):
    """
    This resource represents a long-running operation that is the result of a
    network API call.
    """

    name: str = betterproto.string_field(1)
    """
    The server-assigned name, which is only unique within the same service that
    originally returns it. If you use the default HTTP mapping, the
    `name` should be a resource name ending with `operations/{unique_id}`.
    """

    metadata: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(2)
    """
    Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.
    """

    done: bool = betterproto.bool_field(3)
    """
    If the value is `false`, it means the operation is still in progress.
    If `true`, the operation is completed, and either `error` or `response` is
    available.
    """

    error: "_rpc__.Status" = betterproto.message_field(4, group="result")
    """
    The error result of the operation in case of failure or cancellation.
    """

    response: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(
        5, group="result"
    )
    """
    The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx`
    is the original method name. For example, if the original method name
    is `TakeSnapshot()`, the inferred response type is
    `TakeSnapshotResponse`.
    """


@dataclass(eq=False, repr=False)
class GetOperationRequest(betterproto.Message):
    """The request message for GetOperation."""

    name: str = betterproto.string_field(1)
    """The name of the operation resource."""


import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from .. import rpc as _rpc__



"""
Async-related utilities
"""

import abc
import asyncio
import contextvars
import functools
import itertools
import contextlib
import pathlib
import queue
import os.path
import inspect
import sys
import threading
from concurrent.futures import ThreadPoolExecutor
from weakref import WeakSet, WeakKeyDictionary

from greenlet import greenlet


def create_task(awaitable, name=None):
    if isinstance(awaitable, asyncio.Task):
        task = awaitable
    else:
        task = asyncio.create_task(awaitable)
    if name is None:
        name = getattr(awaitable, "__qualname__", None)
    task.name = name
    return task


def _close_loop(loop):
    if loop is not None:
        try:
            loop.run_until_complete(loop.shutdown_asyncgens())
            try:
                shutdown_default_executor = loop.shutdown_default_executor
            except AttributeError:
                pass
            else:
                loop.run_until_complete(shutdown_default_executor())
        finally:
            loop.close()


class AsyncManager:
    def __init__(self):
        self.task_tree = dict()
        self.resources = dict()

    def track_access(self, access):
        """
        Register the given ``access`` to have been handled by the current
        async task.

        :param access: Access that were done.
        :type access: ConcurrentAccessBase

        This allows :func:`concurrently` to check that concurrent tasks did not
        step on each other's toes.
        """
        try:
            task = asyncio.current_task()
        except RuntimeError:
            pass
        else:
            self.resources.setdefault(task, set()).add(access)

    async def concurrently(self, awaitables):
        """
        Await concurrently for the given awaitables, and cancel them as soon as
        one raises an exception.
        """
        awaitables = list(awaitables)

        # Avoid creating asyncio.Tasks when it's not necessary, as it will
        # disable a the blocking path optimization of Target._execute_async()
        # that uses blocking calls as long as there is only one asyncio.Task
        # running on the event loop.
        if len(awaitables) == 1:
            return [await awaitables[0]]

        tasks = list(map(create_task, awaitables))

        current_task = asyncio.current_task()
        task_tree = self.task_tree

        try:
            node = task_tree[current_task]
        except KeyError:
            is_root_task = True
            node = set()
        else:
            is_root_task = False
        task_tree[current_task] = node

        task_tree.update({child: set() for child in tasks})
        node.update(tasks)

        try:
            return await asyncio.gather(*tasks)
        except BaseException:
            for task in tasks:
                task.cancel()
                try:
                    await task
                except asyncio.CancelledError:
                    pass
            raise
        finally:

            def get_children(task):
                immediate_children = task_tree[task]
                return frozenset(
                    itertools.chain(
                        [task],
                        immediate_children,
                        itertools.chain.from_iterable(map(get_children, immediate_children)),
                    )
                )

            # Get the resources created during the execution of each subtask
            # (directly or indirectly)
            resources = {
                task: frozenset(
                    itertools.chain.from_iterable(self.resources.get(child, []) for child in get_children(task))
                )
                for task in tasks
            }
            for (task1, resources1), (task2, resources2) in itertools.combinations(resources.items(), 2):
                for res1, res2 in itertools.product(resources1, resources2):
                    if issubclass(res2.__class__, res1.__class__) and res1.overlap_with(res2):
                        raise RuntimeError(
                            "Overlapping resources manipulated in concurrent async tasks: {} (task {}) and {} (task {})".format(
                                res1, task1.name, res2, task2.name
                            )
                        )

            if is_root_task:
                self.resources.clear()
                task_tree.clear()

    async def map_concurrently(self, f, keys):
        """
        Similar to :meth:`concurrently`,
        but maps the given function ``f`` on the given ``keys``.

        :return: A dictionary with ``keys`` as keys, and function result as
            values.
        """
        keys = list(keys)
        return dict(zip(keys, await self.concurrently(map(f, keys))))


def compose(*coros):
    """
    Compose coroutines, feeding the output of each as the input of the next
    one.

    ``await compose(f, g)(x)`` is equivalent to ``await f(await g(x))``

    .. note:: In Haskell, ``compose f g h`` would be equivalent to ``f <=< g <=< h``
    """

    async def f(*args, **kwargs):
        empty_dict = {}
        for coro in reversed(coros):
            x = coro(*args, **kwargs)
            # Allow mixing corountines and regular functions
            if asyncio.isfuture(x):
                x = await x
            args = [x]
            kwargs = empty_dict

        return x

    return f


class _AsyncPolymorphicFunction:
    """
    A callable that allows exposing both a synchronous and asynchronous API.

    When called, the blocking synchronous operation is called. The ```asyn``
    attribute gives access to the asynchronous version of the function, and all
    the other attribute access will be redirected to the async function.
    """

    def __init__(self, asyn, blocking):
        self.asyn = asyn
        self.blocking = blocking
        functools.update_wrapper(self, asyn)

    def __get__(self, *args, **kwargs):
        return self.__class__(
            asyn=self.asyn.__get__(*args, **kwargs),
            blocking=self.blocking.__get__(*args, **kwargs),
        )

    # Ensure inspect.iscoroutinefunction() does not detect us as being async,
    # since __call__ is not.
    @property
    def __code__(self):
        return self.__call__.__code__

    def __call__(self, *args, **kwargs):
        return self.blocking(*args, **kwargs)

    def __getattr__(self, attr):
        return getattr(self.asyn, attr)


class memoized_method:
    """
    Decorator to memmoize a method.

    It works for:

        * async methods (coroutine functions)
        * non-async methods
        * method already decorated with :func:`devlib.asyn.asyncf`.

    .. note:: This decorator does not rely on hacks to hash unhashable data. If
        such input is required, it will either have to be coerced to a hashable
        first (e.g. converting a list to a tuple), or the code of
        :func:`devlib.asyn.memoized_method` will have to be updated to do so.
    """

    def __init__(self, f):
        memo = self

        sig = inspect.signature(f)

        def bind(self, *args, **kwargs):
            bound = sig.bind(self, *args, **kwargs)
            bound.apply_defaults()
            key = (bound.args[1:], tuple(sorted(bound.kwargs.items())))

            return (key, bound.args, bound.kwargs)

        def get_cache(self):
            try:
                cache = self.__dict__[memo.name]
            except KeyError:
                cache = {}
                self.__dict__[memo.name] = cache
            return cache

        if inspect.iscoroutinefunction(f):

            @functools.wraps(f)
            async def wrapper(self, *args, **kwargs):
                cache = get_cache(self)
                key, args, kwargs = bind(self, *args, **kwargs)
                try:
                    return cache[key]
                except KeyError:
                    x = await f(*args, **kwargs)
                    cache[key] = x
                    return x

        else:

            @functools.wraps(f)
            def wrapper(self, *args, **kwargs):
                cache = get_cache(self)
                key, args, kwargs = bind(self, *args, **kwargs)
                try:
                    return cache[key]
                except KeyError:
                    x = f(*args, **kwargs)
                    cache[key] = x
                    return x

        self.f = wrapper
        self._name = f.__name__

    @property
    def name(self):
        return "__memoization_cache_of_" + self._name

    def __call__(self, *args, **kwargs):
        return self.f(*args, **kwargs)

    def __get__(self, obj, owner=None):
        return self.f.__get__(obj, owner)

    def __set__(self, obj, value):
        raise RuntimeError("Cannot monkey-patch a memoized function")

    def __set_name__(self, owner, name):
        self._name = name


class _Genlet(greenlet):
    """
    Generator-like object based on ``greenlets``. It allows nested :class:`_Genlet`
    to make their parent yield on their behalf, as if callees could decide to
    be annotated ``yield from`` without modifying the caller.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Forward the context variables to the greenlet, which will not happen
        # by default:
        # https://greenlet.readthedocs.io/en/latest/contextvars.html
        self.gr_context = contextvars.copy_context()

    @classmethod
    def from_coro(cls, coro):
        """
        Create a :class:`_Genlet` from a given coroutine, treating it as a
        generator.
        """
        f = lambda value: self.consume_coro(coro, value)
        self = cls(f)
        return self

    def consume_coro(self, coro, value):
        """
        Send ``value`` to ``coro`` then consume the coroutine, passing all its
        yielded actions to the enclosing :class:`_Genlet`. This allows crossing
        blocking calls layers as if they were async calls with `await`.
        """
        excep = None
        while True:
            try:
                if excep is None:
                    future = coro.send(value)
                else:
                    future = coro.throw(excep)

            except StopIteration as e:
                return e.value
            else:
                parent = self.parent
                # Switch back to the consumer that returns the values via
                # send()
                try:
                    value = parent.switch(future)
                except BaseException as e:
                    excep = e
                    value = None
                else:
                    excep = None

    @classmethod
    def get_enclosing(cls):
        """
        Get the immediately enclosing :class:`_Genlet` in the callstack or
        ``None``.
        """
        g = greenlet.getcurrent()
        while not (isinstance(g, cls) or g is None):
            g = g.parent
        return g

    def _send_throw(self, value, excep):
        self.parent = greenlet.getcurrent()

        # Switch back to the function yielding values
        if excep is None:
            result = self.switch(value)
        else:
            result = self.throw(excep)

        if self:
            return result
        else:
            raise StopIteration(result)

    def gen_send(self, x):
        """
        Similar to generators' ``send`` method.
        """
        return self._send_throw(x, None)

    def gen_throw(self, x):
        """
        Similar to generators' ``throw`` method.
        """
        return self._send_throw(None, x)


class _AwaitableGenlet:
    """
    Wrap a coroutine with a :class:`_Genlet` and wrap that to be awaitable.
    """

    @classmethod
    def wrap_coro(cls, coro):
        async def coro_f():
            # Make sure every new task will be instrumented since a task cannot
            # yield futures on behalf of another task. If that were to happen,
            # the task B trying to do a nested yield would switch back to task
            # A, asking to yield on its behalf. Since the event loop would be
            # currently handling task B, nothing would handle task A trying to
            # yield on behalf of B, leading to a deadlock.
            loop = asyncio.get_running_loop()
            _install_task_factory(loop)

            # Create a top-level _AwaitableGenlet that all nested runs will use
            # to yield their futures
            _coro = cls(coro)

            return await _coro

        return coro_f()

    def __init__(self, coro):
        self._coro = coro

    def __await__(self):
        coro = self._coro
        is_started = inspect.iscoroutine(coro) and coro.cr_running

        def genf():
            gen = _Genlet.from_coro(coro)
            value = None
            excep = None

            # The coroutine is already started, so we need to dispatch the
            # value from the upcoming send() to the gen without running
            # gen first.
            if is_started:
                try:
                    value = yield
                except BaseException as e:
                    excep = e

            while True:
                try:
                    if excep is None:
                        future = gen.gen_send(value)
                    else:
                        future = gen.gen_throw(excep)
                except StopIteration as e:
                    return e.value
                finally:
                    _set_current_context(gen.gr_context)

                try:
                    value = yield future
                except BaseException as e:
                    excep = e
                    value = None
                else:
                    excep = None

        gen = genf()
        if is_started:
            # Start the generator so it waits at the first yield point
            gen.gen_send(None)

        return gen


def _allow_nested_run(coro):
    if _Genlet.get_enclosing() is None:
        return _AwaitableGenlet.wrap_coro(coro)
    else:
        return coro


def allow_nested_run(coro):
    """
    Wrap the coroutine ``coro`` such that nested calls to :func:`run` will be
    allowed.

    .. warning:: The coroutine needs to be consumed in the same OS thread it
        was created in.
    """
    return _allow_nested_run(coro)


# This thread runs coroutines that cannot be ran on the event loop in the
# current thread. Instead, they are scheduled in a separate thread where
# another event loop has been setup, so we can wrap coroutines before
# dispatching them there.
_CORO_THREAD_EXECUTOR = ThreadPoolExecutor(
    # Allow for a ridiculously large number so that we will never end up
    # queuing one job after another. This is critical as we could otherwise end
    # up in deadlock, if a job triggers another job and waits for it.
    max_workers=2
    ** 64,
)


def _check_executor_alive(executor):
    try:
        executor.submit(lambda: None)
    except RuntimeError:
        return False
    else:
        return True


_PATCHED_LOOP_LOCK = threading.Lock()
_PATCHED_LOOP = WeakSet()


def _install_task_factory(loop):
    """
    Install a task factory on the given event ``loop`` so that top-level
    coroutines are wrapped using :func:`allow_nested_run`. This ensures that
    the nested :func:`run` infrastructure will be available.
    """

    def install(loop):
        if sys.version_info >= (3, 11):

            def default_factory(loop, coro, context=None):
                return asyncio.Task(coro, loop=loop, context=context)

        else:

            def default_factory(loop, coro, context=None):
                return asyncio.Task(coro, loop=loop)

        make_task = loop.get_task_factory() or default_factory

        def factory(loop, coro, context=None):
            # Make sure each Task will be able to yield on behalf of its nested
            # await beneath blocking layers
            coro = _AwaitableGenlet.wrap_coro(coro)
            return make_task(loop, coro, context=context)

        loop.set_task_factory(factory)

    with _PATCHED_LOOP_LOCK:
        if loop in _PATCHED_LOOP:
            return
        else:
            install(loop)
            _PATCHED_LOOP.add(loop)


def _set_current_context(ctx):
    """
    Get all the variable from the passed ``ctx`` and set them in the current
    context.
    """
    for var, val in ctx.items():
        var.set(val)


class _CoroRunner(abc.ABC):
    """
    ABC for an object that can execute multiple coroutines in a given
    environment.

    This allows running coroutines for which it might be an assumption, such as
    the awaitables yielded by an async generator that are all attached to a
    single event loop.
    """

    @abc.abstractmethod
    def _run(self, coro):
        pass

    def run(self, coro):
        # Ensure we have a fresh coroutine. inspect.getcoroutinestate() does not
        # work on all objects that asyncio creates on some version of Python, such
        # as iterable_coroutine
        assert not (inspect.iscoroutine(coro) and coro.cr_running)
        return self._run(coro)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, tb):
        pass


class _ThreadCoroRunner(_CoroRunner):
    """
    Run the coroutines on a thread picked from a
    :class:`concurrent.futures.ThreadPoolExecutor`.

    Critically, this allows running multiple coroutines out of the same thread,
    which will be reserved until the runner ``__exit__`` method is called.
    """

    def __init__(self, future, jobq, resq):
        self._future = future
        self._jobq = jobq
        self._resq = resq

    @staticmethod
    def _thread_f(jobq, resq):
        def handle_jobs(runner):
            while True:
                job = jobq.get()
                if job is None:
                    return
                else:
                    ctx, coro = job
                    try:
                        value = ctx.run(runner.run, coro)
                    except BaseException as e:
                        value = None
                        excep = e
                    else:
                        excep = None

                    resq.put((ctx, excep, value))

        with _LoopCoroRunner(None) as runner:
            handle_jobs(runner)

    @classmethod
    def from_executor(cls, executor):
        jobq = queue.SimpleQueue()
        resq = queue.SimpleQueue()

        try:
            future = executor.submit(cls._thread_f, jobq, resq)
        except RuntimeError as e:
            if _check_executor_alive(executor):
                raise e
            else:
                raise RuntimeError(
                    "Devlib relies on nested asyncio implementation requiring threads. These threads are not available while shutting down the interpreter."
                )

        return cls(
            jobq=jobq,
            resq=resq,
            future=future,
        )

    def _run(self, coro):
        ctx = contextvars.copy_context()
        self._jobq.put((ctx, coro))
        ctx, excep, value = self._resq.get()

        _set_current_context(ctx)

        if excep is None:
            return value
        else:
            raise excep

    def __exit__(self, *args, **kwargs):
        self._jobq.put(None)
        self._future.result()


class _LoopCoroRunner(_CoroRunner):
    """
    Run a coroutine on the given event loop.

    The passed event loop is assumed to not be running. If ``None`` is passed,
    a new event loop will be created in ``__enter__`` and closed in
    ``__exit__``.
    """

    def __init__(self, loop):
        self.loop = loop
        self._owned = False

    def _run(self, coro):
        loop = self.loop

        # Back-propagate the contextvars that could have been modified by the
        # coroutine. This could be handled by asyncio.Runner().run(...,
        # context=...) or loop.create_task(..., context=...) but these APIs are
        # only available since Python 3.11
        ctx = None

        async def capture_ctx():
            nonlocal ctx
            try:
                return await _allow_nested_run(coro)
            finally:
                ctx = contextvars.copy_context()

        try:
            return loop.run_until_complete(capture_ctx())
        finally:
            _set_current_context(ctx)

    def __enter__(self):
        loop = self.loop
        if loop is None:
            owned = True
            loop = asyncio.new_event_loop()
        else:
            owned = False

        asyncio.set_event_loop(loop)

        self.loop = loop
        self._owned = owned
        return self

    def __exit__(self, *args, **kwargs):
        if self._owned:
            asyncio.set_event_loop(None)
            _close_loop(self.loop)


class _GenletCoroRunner(_CoroRunner):
    """
    Run a coroutine assuming one of the parent coroutines was wrapped with
    :func:`allow_nested_run`.
    """

    def __init__(self, g):
        self._g = g

    def _run(self, coro):
        return self._g.consume_coro(coro, None)


def _get_runner():
    executor = _CORO_THREAD_EXECUTOR
    g = _Genlet.get_enclosing()
    try:
        loop = asyncio.get_running_loop()
    except RuntimeError:
        loop = None

    # We have an coroutine wrapped with allow_nested_run() higher in the
    # callstack, that we will be able to use as a conduit to yield the
    # futures.
    if g is not None:
        return _GenletCoroRunner(g)
    # No event loop setup, so we can just make our own
    elif loop is None:
        return _LoopCoroRunner(None)
    # There is an event loop setup, but it is not currently running so we
    # can just re-use it.
    #
    # TODO: for now, this path is dead since asyncio.get_running_loop() will
    # always raise a RuntimeError if the loop is not running, even if
    # asyncio.set_event_loop() was used.
    elif not loop.is_running():
        return _LoopCoroRunner(loop)
    # There is an event loop currently running in our thread, so we cannot
    # just create another event loop and install it since asyncio forbids
    # that. The only choice is doing this in a separate thread that we
    # fully control.
    else:
        return _ThreadCoroRunner.from_executor(executor)


def run(coro):
    """
    Similar to :func:`asyncio.run` but can be called while an event loop is
    running if a coroutine higher in the callstack has been wrapped using
    :func:`allow_nested_run`.

    Note that context variables from :mod:`contextvars` will be available in
    the coroutine, and unlike with :func:`asyncio.run`, any update to them will
    be reflected in the context of the caller. This allows context variable
    updates to cross an arbitrary number of run layers, as if all those layers
    were just part of the same coroutine.
    """
    runner = _get_runner()
    with runner as runner:
        return runner.run(coro)


def asyncf(f):
    """
    Decorator used to turn a coroutine into a blocking function, with an
    optional asynchronous API.

    **Example**::

        @asyncf
        async def foo(x):
            await do_some_async_things(x)
            return x

        # Blocking call, just as if the function was synchronous, except it may
        # use asynchronous code inside, e.g. to do concurrent operations.
        foo(42)

        # Asynchronous API, foo.asyn being a corountine
        await foo.asyn(42)

    This allows the same implementation to be both used as blocking for ease of
    use and backward compatibility, or exposed as a corountine for callers that
    can deal with awaitables.
    """

    @functools.wraps(f)
    def blocking(*args, **kwargs):
        # Since run() needs a corountine, make sure we provide one
        async def wrapper():
            x = f(*args, **kwargs)
            # Async generators have to be consumed and accumulated in a list
            # before crossing a blocking boundary.
            if inspect.isasyncgen(x):

                def genf():
                    asyncgen = x.__aiter__()
                    while True:
                        try:
                            yield run(asyncgen.__anext__())
                        except StopAsyncIteration:
                            return

                return genf()
            else:
                return await x

        return run(wrapper())

    return _AsyncPolymorphicFunction(
        asyn=f,
        blocking=blocking,
    )


class _AsyncPolymorphicCMState:
    def __init__(self):
        self.nesting = 0
        self.runner = None

    def _update_nesting(self, n):
        x = self.nesting
        assert x >= 0
        x = x + n
        self.nesting = x
        return bool(x)

    def _get_runner(self):
        runner = self.runner
        if runner is None:
            assert not self.nesting
            runner = _get_runner()
            runner.__enter__()
        self.runner = runner
        return runner

    def _cleanup_runner(self, force=False):
        def cleanup():
            self.runner = None
            if runner is not None:
                runner.__exit__(None, None, None)

        runner = self.runner
        if force:
            cleanup()
        else:
            assert runner is not None
            if not self._update_nesting(0):
                cleanup()


class _AsyncPolymorphicCM:
    """
    Wrap an async context manager such that it exposes a synchronous API as
    well for backward compatibility.
    """

    def __init__(self, async_cm):
        self.cm = async_cm
        self._state = threading.local()

    def _get_state(self):
        try:
            return self._state.x
        except AttributeError:
            state = _AsyncPolymorphicCMState()
            self._state.x = state
            return state

    def _delete_state(self):
        try:
            del self._state.x
        except AttributeError:
            pass

    def __aenter__(self, *args, **kwargs):
        return self.cm.__aenter__(*args, **kwargs)

    def __aexit__(self, *args, **kwargs):
        return self.cm.__aexit__(*args, **kwargs)

    @staticmethod
    def _exit(state):
        state._update_nesting(-1)
        state._cleanup_runner()

    def __enter__(self, *args, **kwargs):
        state = self._get_state()
        runner = state._get_runner()

        # Increase the nesting count _before_ we start running the
        # coroutine, in case it is a recursive context manager
        state._update_nesting(1)

        try:
            coro = self.cm.__aenter__(*args, **kwargs)
            return runner.run(coro)
        except BaseException:
            self._exit(state)
            raise

    def __exit__(self, *args, **kwargs):
        coro = self.cm.__aexit__(*args, **kwargs)

        state = self._get_state()
        runner = state._get_runner()

        try:
            return runner.run(coro)
        finally:
            self._exit(state)

    def __del__(self):
        self._get_state()._cleanup_runner(force=True)


def asynccontextmanager(f):
    """
    Same as :func:`contextlib.asynccontextmanager` except that it can also be
    used with a regular ``with`` statement for backward compatibility.
    """
    f = contextlib.asynccontextmanager(f)

    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        cm = f(*args, **kwargs)
        return _AsyncPolymorphicCM(cm)

    return wrapper


class ConcurrentAccessBase(abc.ABC):
    """
    Abstract Base Class for resources tracked by :func:`concurrently`.
    """

    @abc.abstractmethod
    def overlap_with(self, other):
        """
        Return ``True`` if the resource overlaps with the given one.

        :param other: Resources that should not overlap with ``self``.
        :type other: devlib.utils.asym.ConcurrentAccessBase

        .. note:: It is guaranteed that ``other`` will be a subclass of our
            class.
        """


class PathAccess(ConcurrentAccessBase):
    """
    Concurrent resource representing a file access.

    :param namespace: Identifier of the namespace of the path. One of "target" or "host".
    :type namespace: str

    :param path: Normalized path to the file.
    :type path: str

    :param mode: Opening mode of the file. Can be ``"r"`` for read and ``"w"``
        for writing.
    :type mode: str
    """

    def __init__(self, namespace, path, mode):
        assert namespace in ("host", "target")
        self.namespace = namespace
        assert mode in ("r", "w")
        self.mode = mode
        self.path = os.path.abspath(path) if namespace == "host" else os.path.normpath(path)

    def overlap_with(self, other):
        path1 = pathlib.Path(self.path).resolve()
        path2 = pathlib.Path(other.path).resolve()
        return (
            self.namespace == other.namespace
            and "w" in (self.mode, other.mode)
            and (path1 == path2 or path1 in path2.parents or path2 in path1.parents)
        )

    def __str__(self):
        mode = {
            "r": "read",
            "w": "write",
        }[self.mode]
        return "{} ({})".format(self.path, mode)



from . import billing_pb2 as _billing_pb2
from . import deployed_model_pb2 as _deployed_model_pb2
from . import deployment_pb2 as _deployment_pb2
from . import options_pb2 as _options_pb2
from . import status_pb2 as _status_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from ..google.api import resource_pb2 as _resource_pb2
from ..google.api import visibility_pb2 as _visibility_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from ..google.type import date_pb2 as _date_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Model(_message.Message):
    __slots__ = ("name", "display_name", "description", "create_time", "created_by", "state", "status", "kind", "github_url", "hugging_face_url", "base_model_details", "peft_details", "teft_details", "public", "conversation_config", "context_length", "supports_image_input", "supports_tools", "imported_from", "tokens_per_second", "featured_priority", "fine_tuning_job", "sku_infos", "default_draft_model", "default_draft_token_count", "precisions", "deployed_model_refs", "cluster", "deprecation_date", "calibrated", "tunable", "supports_lora", "use_hf_apply_chat_template", "extra_deployment_args", "update_time", "default_sampling_params", "gcs_uri", "rl_tunable")
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATE_UNSPECIFIED: _ClassVar[Model.State]
        UPLOADING: _ClassVar[Model.State]
        READY: _ClassVar[Model.State]
    STATE_UNSPECIFIED: Model.State
    UPLOADING: Model.State
    READY: Model.State
    class Kind(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        KIND_UNSPECIFIED: _ClassVar[Model.Kind]
        HF_BASE_MODEL: _ClassVar[Model.Kind]
        HF_PEFT_ADDON: _ClassVar[Model.Kind]
        HF_TEFT_ADDON: _ClassVar[Model.Kind]
        FLUMINA_BASE_MODEL: _ClassVar[Model.Kind]
        FLUMINA_ADDON: _ClassVar[Model.Kind]
        DRAFT_ADDON: _ClassVar[Model.Kind]
        FIRE_AGENT: _ClassVar[Model.Kind]
        LIVE_MERGE: _ClassVar[Model.Kind]
    KIND_UNSPECIFIED: Model.Kind
    HF_BASE_MODEL: Model.Kind
    HF_PEFT_ADDON: Model.Kind
    HF_TEFT_ADDON: Model.Kind
    FLUMINA_BASE_MODEL: Model.Kind
    FLUMINA_ADDON: Model.Kind
    DRAFT_ADDON: Model.Kind
    FIRE_AGENT: Model.Kind
    LIVE_MERGE: Model.Kind
    class DefaultSamplingParamsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: float
        def __init__(self, key: _Optional[str] = ..., value: _Optional[float] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    KIND_FIELD_NUMBER: _ClassVar[int]
    GITHUB_URL_FIELD_NUMBER: _ClassVar[int]
    HUGGING_FACE_URL_FIELD_NUMBER: _ClassVar[int]
    BASE_MODEL_DETAILS_FIELD_NUMBER: _ClassVar[int]
    PEFT_DETAILS_FIELD_NUMBER: _ClassVar[int]
    TEFT_DETAILS_FIELD_NUMBER: _ClassVar[int]
    PUBLIC_FIELD_NUMBER: _ClassVar[int]
    CONVERSATION_CONFIG_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_LENGTH_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_IMAGE_INPUT_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_TOOLS_FIELD_NUMBER: _ClassVar[int]
    IMPORTED_FROM_FIELD_NUMBER: _ClassVar[int]
    TOKENS_PER_SECOND_FIELD_NUMBER: _ClassVar[int]
    FEATURED_PRIORITY_FIELD_NUMBER: _ClassVar[int]
    FINE_TUNING_JOB_FIELD_NUMBER: _ClassVar[int]
    SKU_INFOS_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_DRAFT_MODEL_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_DRAFT_TOKEN_COUNT_FIELD_NUMBER: _ClassVar[int]
    PRECISIONS_FIELD_NUMBER: _ClassVar[int]
    DEPLOYED_MODEL_REFS_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_FIELD_NUMBER: _ClassVar[int]
    DEPRECATION_DATE_FIELD_NUMBER: _ClassVar[int]
    CALIBRATED_FIELD_NUMBER: _ClassVar[int]
    TUNABLE_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_LORA_FIELD_NUMBER: _ClassVar[int]
    USE_HF_APPLY_CHAT_TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    EXTRA_DEPLOYMENT_ARGS_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_SAMPLING_PARAMS_FIELD_NUMBER: _ClassVar[int]
    GCS_URI_FIELD_NUMBER: _ClassVar[int]
    RL_TUNABLE_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    description: str
    create_time: _timestamp_pb2.Timestamp
    created_by: str
    state: Model.State
    status: _status_pb2.Status
    kind: Model.Kind
    github_url: str
    hugging_face_url: str
    base_model_details: BaseModelDetails
    peft_details: PEFTDetails
    teft_details: TEFTDetails
    public: bool
    conversation_config: ConversationConfig
    context_length: int
    supports_image_input: bool
    supports_tools: bool
    imported_from: str
    tokens_per_second: int
    featured_priority: int
    fine_tuning_job: str
    sku_infos: _containers.RepeatedCompositeFieldContainer[_billing_pb2.SKUInfo]
    default_draft_model: str
    default_draft_token_count: int
    precisions: _containers.RepeatedScalarFieldContainer[_deployment_pb2.Deployment.Precision]
    deployed_model_refs: _containers.RepeatedCompositeFieldContainer[_deployed_model_pb2.DeployedModelRef]
    cluster: str
    deprecation_date: _date_pb2.Date
    calibrated: bool
    tunable: bool
    supports_lora: bool
    use_hf_apply_chat_template: bool
    extra_deployment_args: _containers.RepeatedScalarFieldContainer[str]
    update_time: _timestamp_pb2.Timestamp
    default_sampling_params: _containers.ScalarMap[str, float]
    gcs_uri: str
    rl_tunable: bool
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., description: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., state: _Optional[_Union[Model.State, str]] = ..., status: _Optional[_Union[_status_pb2.Status, _Mapping]] = ..., kind: _Optional[_Union[Model.Kind, str]] = ..., github_url: _Optional[str] = ..., hugging_face_url: _Optional[str] = ..., base_model_details: _Optional[_Union[BaseModelDetails, _Mapping]] = ..., peft_details: _Optional[_Union[PEFTDetails, _Mapping]] = ..., teft_details: _Optional[_Union[TEFTDetails, _Mapping]] = ..., public: bool = ..., conversation_config: _Optional[_Union[ConversationConfig, _Mapping]] = ..., context_length: _Optional[int] = ..., supports_image_input: bool = ..., supports_tools: bool = ..., imported_from: _Optional[str] = ..., tokens_per_second: _Optional[int] = ..., featured_priority: _Optional[int] = ..., fine_tuning_job: _Optional[str] = ..., sku_infos: _Optional[_Iterable[_Union[_billing_pb2.SKUInfo, _Mapping]]] = ..., default_draft_model: _Optional[str] = ..., default_draft_token_count: _Optional[int] = ..., precisions: _Optional[_Iterable[_Union[_deployment_pb2.Deployment.Precision, str]]] = ..., deployed_model_refs: _Optional[_Iterable[_Union[_deployed_model_pb2.DeployedModelRef, _Mapping]]] = ..., cluster: _Optional[str] = ..., deprecation_date: _Optional[_Union[_date_pb2.Date, _Mapping]] = ..., calibrated: bool = ..., tunable: bool = ..., supports_lora: bool = ..., use_hf_apply_chat_template: bool = ..., extra_deployment_args: _Optional[_Iterable[str]] = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., default_sampling_params: _Optional[_Mapping[str, float]] = ..., gcs_uri: _Optional[str] = ..., rl_tunable: bool = ...) -> None: ...

class BaseModelDetails(_message.Message):
    __slots__ = ("world_size", "checkpoint_format", "huggingface_files", "parameter_count", "moe", "tunable", "model_type", "supports_fireattention")
    class CheckpointFormat(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        CHECKPOINT_FORMAT_UNSPECIFIED: _ClassVar[BaseModelDetails.CheckpointFormat]
        NATIVE: _ClassVar[BaseModelDetails.CheckpointFormat]
        HUGGINGFACE: _ClassVar[BaseModelDetails.CheckpointFormat]
    CHECKPOINT_FORMAT_UNSPECIFIED: BaseModelDetails.CheckpointFormat
    NATIVE: BaseModelDetails.CheckpointFormat
    HUGGINGFACE: BaseModelDetails.CheckpointFormat
    WORLD_SIZE_FIELD_NUMBER: _ClassVar[int]
    CHECKPOINT_FORMAT_FIELD_NUMBER: _ClassVar[int]
    HUGGINGFACE_FILES_FIELD_NUMBER: _ClassVar[int]
    PARAMETER_COUNT_FIELD_NUMBER: _ClassVar[int]
    MOE_FIELD_NUMBER: _ClassVar[int]
    TUNABLE_FIELD_NUMBER: _ClassVar[int]
    MODEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    SUPPORTS_FIREATTENTION_FIELD_NUMBER: _ClassVar[int]
    world_size: int
    checkpoint_format: BaseModelDetails.CheckpointFormat
    huggingface_files: _containers.RepeatedScalarFieldContainer[str]
    parameter_count: int
    moe: bool
    tunable: bool
    model_type: str
    supports_fireattention: bool
    def __init__(self, world_size: _Optional[int] = ..., checkpoint_format: _Optional[_Union[BaseModelDetails.CheckpointFormat, str]] = ..., huggingface_files: _Optional[_Iterable[str]] = ..., parameter_count: _Optional[int] = ..., moe: bool = ..., tunable: bool = ..., model_type: _Optional[str] = ..., supports_fireattention: bool = ...) -> None: ...

class PEFTDetails(_message.Message):
    __slots__ = ("base_model", "r", "target_modules", "base_model_type", "merge_addon_model_name")
    BASE_MODEL_FIELD_NUMBER: _ClassVar[int]
    R_FIELD_NUMBER: _ClassVar[int]
    TARGET_MODULES_FIELD_NUMBER: _ClassVar[int]
    BASE_MODEL_TYPE_FIELD_NUMBER: _ClassVar[int]
    MERGE_ADDON_MODEL_NAME_FIELD_NUMBER: _ClassVar[int]
    base_model: str
    r: int
    target_modules: _containers.RepeatedScalarFieldContainer[str]
    base_model_type: str
    merge_addon_model_name: str
    def __init__(self, base_model: _Optional[str] = ..., r: _Optional[int] = ..., target_modules: _Optional[_Iterable[str]] = ..., base_model_type: _Optional[str] = ..., merge_addon_model_name: _Optional[str] = ...) -> None: ...

class TEFTDetails(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ConversationConfig(_message.Message):
    __slots__ = ("style", "system", "template")
    STYLE_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_FIELD_NUMBER: _ClassVar[int]
    TEMPLATE_FIELD_NUMBER: _ClassVar[int]
    style: str
    system: str
    template: str
    def __init__(self, style: _Optional[str] = ..., system: _Optional[str] = ..., template: _Optional[str] = ...) -> None: ...

class CreateModelRequest(_message.Message):
    __slots__ = ("parent", "model", "model_id", "cluster")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    MODEL_ID_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_FIELD_NUMBER: _ClassVar[int]
    parent: str
    model: Model
    model_id: str
    cluster: str
    def __init__(self, parent: _Optional[str] = ..., model: _Optional[_Union[Model, _Mapping]] = ..., model_id: _Optional[str] = ..., cluster: _Optional[str] = ...) -> None: ...

class GetModelRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class PrepareModelRequest(_message.Message):
    __slots__ = ("name", "precision")
    NAME_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    name: str
    precision: _deployment_pb2.Deployment.Precision
    def __init__(self, name: _Optional[str] = ..., precision: _Optional[_Union[_deployment_pb2.Deployment.Precision, str]] = ...) -> None: ...

class GetModelUploadEndpointRequest(_message.Message):
    __slots__ = ("name", "filename_to_size", "enable_resumable_upload")
    class FilenameToSizeEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: int
        def __init__(self, key: _Optional[str] = ..., value: _Optional[int] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    FILENAME_TO_SIZE_FIELD_NUMBER: _ClassVar[int]
    ENABLE_RESUMABLE_UPLOAD_FIELD_NUMBER: _ClassVar[int]
    name: str
    filename_to_size: _containers.ScalarMap[str, int]
    enable_resumable_upload: bool
    def __init__(self, name: _Optional[str] = ..., filename_to_size: _Optional[_Mapping[str, int]] = ..., enable_resumable_upload: bool = ...) -> None: ...

class GetModelUploadEndpointResponse(_message.Message):
    __slots__ = ("filename_to_signed_urls", "filename_to_unsigned_uris")
    class FilenameToSignedUrlsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class FilenameToUnsignedUrisEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    FILENAME_TO_SIGNED_URLS_FIELD_NUMBER: _ClassVar[int]
    FILENAME_TO_UNSIGNED_URIS_FIELD_NUMBER: _ClassVar[int]
    filename_to_signed_urls: _containers.ScalarMap[str, str]
    filename_to_unsigned_uris: _containers.ScalarMap[str, str]
    def __init__(self, filename_to_signed_urls: _Optional[_Mapping[str, str]] = ..., filename_to_unsigned_uris: _Optional[_Mapping[str, str]] = ...) -> None: ...

class GetModelDownloadEndpointRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class GetModelDownloadEndpointResponse(_message.Message):
    __slots__ = ("filename_to_signed_urls",)
    class FilenameToSignedUrlsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    FILENAME_TO_SIGNED_URLS_FIELD_NUMBER: _ClassVar[int]
    filename_to_signed_urls: _containers.ScalarMap[str, str]
    def __init__(self, filename_to_signed_urls: _Optional[_Mapping[str, str]] = ...) -> None: ...

class ValidateModelUploadRequest(_message.Message):
    __slots__ = ("name", "skip_hf_config_validation")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SKIP_HF_CONFIG_VALIDATION_FIELD_NUMBER: _ClassVar[int]
    name: str
    skip_hf_config_validation: bool
    def __init__(self, name: _Optional[str] = ..., skip_hf_config_validation: bool = ...) -> None: ...

class ListModelsRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter", "order_by", "include_deployed_model_refs")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_DEPLOYED_MODEL_REFS_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    include_deployed_model_refs: bool
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ..., include_deployed_model_refs: bool = ...) -> None: ...

class ListModelsResponse(_message.Message):
    __slots__ = ("models", "next_page_token", "total_size")
    MODELS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    models: _containers.RepeatedCompositeFieldContainer[Model]
    next_page_token: str
    total_size: int
    def __init__(self, models: _Optional[_Iterable[_Union[Model, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class ListServerlessLoraModelsRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ListServerlessLoraModelsResponse(_message.Message):
    __slots__ = ("models",)
    MODELS_FIELD_NUMBER: _ClassVar[int]
    models: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, models: _Optional[_Iterable[str]] = ...) -> None: ...

class ValidateModelConfigRequest(_message.Message):
    __slots__ = ("config_json", "tokenizer_config_json")
    CONFIG_JSON_FIELD_NUMBER: _ClassVar[int]
    TOKENIZER_CONFIG_JSON_FIELD_NUMBER: _ClassVar[int]
    config_json: str
    tokenizer_config_json: str
    def __init__(self, config_json: _Optional[str] = ..., tokenizer_config_json: _Optional[str] = ...) -> None: ...

class UpdateModelRequest(_message.Message):
    __slots__ = ("model", "update_mask")
    MODEL_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    model: Model
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, model: _Optional[_Union[Model, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class DeleteModelRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class AwsS3ModelSource(_message.Message):
    __slots__ = ("s3_bucket", "s3_path", "role_arn", "access_key_id", "access_secret")
    S3_BUCKET_FIELD_NUMBER: _ClassVar[int]
    S3_PATH_FIELD_NUMBER: _ClassVar[int]
    ROLE_ARN_FIELD_NUMBER: _ClassVar[int]
    ACCESS_KEY_ID_FIELD_NUMBER: _ClassVar[int]
    ACCESS_SECRET_FIELD_NUMBER: _ClassVar[int]
    s3_bucket: str
    s3_path: str
    role_arn: str
    access_key_id: str
    access_secret: str
    def __init__(self, s3_bucket: _Optional[str] = ..., s3_path: _Optional[str] = ..., role_arn: _Optional[str] = ..., access_key_id: _Optional[str] = ..., access_secret: _Optional[str] = ...) -> None: ...

class ImportModelRequest(_message.Message):
    __slots__ = ("name", "aws_s3_source")
    NAME_FIELD_NUMBER: _ClassVar[int]
    AWS_S3_SOURCE_FIELD_NUMBER: _ClassVar[int]
    name: str
    aws_s3_source: AwsS3ModelSource
    def __init__(self, name: _Optional[str] = ..., aws_s3_source: _Optional[_Union[AwsS3ModelSource, _Mapping]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/snapshot.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/snapshot.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16gateway/snapshot.proto\x12\x07gateway\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xf9\x03\n\x08Snapshot\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x36\n\x0b\x63reate_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12-\n\x05state\x18\x03 \x01(\x0e\x32\x17.gateway.Snapshot.StateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x04 \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x18\n\timage_ref\x18\x05 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0bupdate_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\"Q\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x0c\n\x08\x43REATING\x10\x01\x12\t\n\x05READY\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0c\n\x08\x44\x45LETING\x10\x04:\xa4\x01\xea\x41\x8f\x01\n\x19\x61pi.fireworks.ai/Snapshot\x12]accounts/{AccountId}/clusters/{ClusterId}/environments/{EnvironmentId}/snapshots/{SnapshotId}*\tsnapshots2\x08snapshot\x82\xf1\x04\r\n\x0b\x45nvironment\"X\n\x15\x43reateSnapshotRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12)\n\x08snapshot\x18\x02 \x01(\x0b\x32\x11.gateway.SnapshotB\x04\xe2\x41\x01\x02\"(\n\x12GetSnapshotRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\"\x8d\x01\n\x14ListSnapshotsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\"j\n\x15ListSnapshotsResponse\x12$\n\tsnapshots\x18\x01 \x03(\x0b\x32\x11.gateway.Snapshot\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"+\n\x15\x44\x65leteSnapshotRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.snapshot_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_SNAPSHOT'].fields_by_name['name']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_SNAPSHOT'].fields_by_name['create_time']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_SNAPSHOT'].fields_by_name['state']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_SNAPSHOT'].fields_by_name['status']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_SNAPSHOT'].fields_by_name['image_ref']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['image_ref']._serialized_options = b'\342A\002\003\005'
  _globals['_SNAPSHOT'].fields_by_name['update_time']._loaded_options = None
  _globals['_SNAPSHOT'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_SNAPSHOT']._loaded_options = None
  _globals['_SNAPSHOT']._serialized_options = b'\352A\217\001\n\031api.fireworks.ai/Snapshot\022]accounts/{AccountId}/clusters/{ClusterId}/environments/{EnvironmentId}/snapshots/{SnapshotId}*\tsnapshots2\010snapshot\202\361\004\r\n\013Environment'
  _globals['_CREATESNAPSHOTREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATESNAPSHOTREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATESNAPSHOTREQUEST'].fields_by_name['snapshot']._loaded_options = None
  _globals['_CREATESNAPSHOTREQUEST'].fields_by_name['snapshot']._serialized_options = b'\342A\001\002'
  _globals['_GETSNAPSHOTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETSNAPSHOTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTSNAPSHOTSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_DELETESNAPSHOTREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETESNAPSHOTREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_SNAPSHOT']._serialized_start=174
  _globals['_SNAPSHOT']._serialized_end=679
  _globals['_SNAPSHOT_STATE']._serialized_start=431
  _globals['_SNAPSHOT_STATE']._serialized_end=512
  _globals['_CREATESNAPSHOTREQUEST']._serialized_start=681
  _globals['_CREATESNAPSHOTREQUEST']._serialized_end=769
  _globals['_GETSNAPSHOTREQUEST']._serialized_start=771
  _globals['_GETSNAPSHOTREQUEST']._serialized_end=811
  _globals['_LISTSNAPSHOTSREQUEST']._serialized_start=814
  _globals['_LISTSNAPSHOTSREQUEST']._serialized_end=955
  _globals['_LISTSNAPSHOTSRESPONSE']._serialized_start=957
  _globals['_LISTSNAPSHOTSRESPONSE']._serialized_end=1063
  _globals['_DELETESNAPSHOTREQUEST']._serialized_start=1065
  _globals['_DELETESNAPSHOTREQUEST']._serialized_end=1108
# @@protoc_insertion_point(module_scope)
